function [] = GlassMapViewer()

% ||
fn = [ShowPath() '\VIS_SWIR_GlassList.mat']; % VIS_SWIR_GlassList.mat must be in the same folder as function/exe
loadedstruct = load(fn);
% ||
% loadedstruct = uiimport;
% ||

VIS_SWIR_GlassList = loadedstruct.VIS_SWIR_GlassList;

% create glass map plotter

% ability to hide catalogs (clickable legend)
% hide/show discontinued glasses
% N vs. V in VIS and SWIR (separate axes)
% have separate axes with linked zoom motion
% clicking on glass labels it in both axes and shows details in separate figure window
% details figure has N vs. Lambda, T vs. Lambda, availability, catalog,
% name, Nd, Vd, Nswir, Vswir

cats = unique(VIS_SWIR_GlassList.Catalog,'stable'); % get unique list of catalogs

name = VIS_SWIR_GlassList.Name;
catalog = VIS_SWIR_GlassList.Catalog;
nd = VIS_SWIR_GlassList.Nd;
vd = VIS_SWIR_GlassList.Vd;
df = VIS_SWIR_GlassList.DispersionFormulaType;
dfc = VIS_SWIR_GlassList.DispersionFormulaConstants;
av = VIS_SWIR_GlassList.Availability;
twav = VIS_SWIR_GlassList.TransmissionWavelengths;
tdat = VIS_SWIR_GlassList.TransmissionData;
ns = VIS_SWIR_GlassList.Nswir;
vs = VIS_SWIR_GlassList.Vswir;
avind = find(av);

% define dispersion equations
dfLAU = @(dfc,lam) sqrt(dfc(:,1) + dfc(:,2).*lam.^2 + dfc(:,3).*lam.^(-2) + dfc(:,4).*lam.^(-4) + dfc(:,5).*lam.^(-6) + dfc(:,6).*lam.^(-8) + dfc(:,7).*lam.^(-10)...
                      + dfc(:,8).*lam.^(-12) + dfc(:,9).*lam.^(-14) + dfc(:,10).*lam.^(-16) + dfc(:,11).*lam.^(-18) + dfc(:,12).*lam.^(-20));
dfGMS = @(dfc,lam) sqrt(1 + dfc(:,1).*lam.^2./(lam.^2-dfc(:,2)) + dfc(:,3).*lam.^2./(lam.^2-dfc(:,4)) + dfc(:,5).*lam.^2./(lam.^2-dfc(:,6))...
                      + dfc(:,7).*lam.^2./(lam.^2-dfc(:,8)) + dfc(:,9).*lam.^2./(lam.^2-dfc(:,10)) + dfc(:,11).*lam.^2./(lam.^2-dfc(:,12)));
dfSLM = @(dfc,lam) sqrt(1 + dfc(:,1).*lam.^2./(lam.^2-dfc(:,2).^2) + dfc(:,3).*lam.^2./(lam.^2-dfc(:,4).^2) + dfc(:,5).*lam.^2./(lam.^2-dfc(:,6).^2)...
                      + dfc(:,7).*lam.^2./(lam.^2-dfc(:,8).^2) + dfc(:,9).*lam.^2./(lam.^2-dfc(:,10).^2) + dfc(:,11).*lam.^2./(lam.^2-dfc(:,12).^2));

wavplot = linspace(0.46,1.7,100);
nplot = zeros(size(name,1),size(wavplot,2));
wavcalc = [0.48613 .58756 0.65627 0.9 1.3 1.7 0.43584]; % wavelengths for dispersion calcs [nF,nd,nC,n1,n1.3,n1.6,ng]
ncalc = zeros(size(nd,1),7);
for gi = 1:size(nplot,1);
    switch df{gi}
        case 'LAU'
            nplot(gi,:) = dfLAU(dfc(gi,:),wavplot);
            ncalc(gi,:) = dfLAU(dfc(gi,:),wavcalc);
        case 'GMS'
            nplot(gi,:) = dfGMS(dfc(gi,:),wavplot);
            ncalc(gi,:) = dfGMS(dfc(gi,:),wavcalc);
        case 'SLM'
            nplot(gi,:) = dfSLM(dfc(gi,:),wavplot);
            ncalc(gi,:) = dfSLM(dfc(gi,:),wavcalc);
    end
end
PdC = (ncalc(:,2)-ncalc(:,3))./(ncalc(:,1)-ncalc(:,3));
Pswir = (ncalc(:,5)-ncalc(:,6))./(ncalc(:,4)-ncalc(:,6));

catglas = cellfun(@(cat) find(strcmp(cat,catalog)),cats,'UniformOutput',false); % get indices of glasses in each catalog
catglasav = cellfun(@(catglas) intersect(catglas,avind),catglas,'UniformOutput',false); % get indices of AVAILABILE glasses in each catalog
catav = find(cellfun(@(cat) ~isempty(cat),catglasav)); % catalog indices that have at least one non-discontinued glass
% note that all CORNFR and PILKINGTON glasses are discontinued

curg = 1; % index of currently labeled glass
disyn = false; % flag if discontinued glasses are shown
glabyn = false; % flag if glass labels and details are shown

ss = get(0,'ScreenSize');

%% control panel figure
fogm = findobj('type','figure','Name','Control Panel');
if isempty(fogm);
    figcon = figure(); % create figure for glassmaps
else
    figure(fogm(1));
    figcon = clf('reset'); % clear previous "Control Panel" figure
end
fs = [900 600 650 390];
set(figcon,'Toolbar','none','CloseRequestFcn',@closefigs,'Name','Control Panel','numbertitle','off','position',[max(0,min(fs(1),ss(3)-fs(3))) max(0,min(fs(2),ss(4)-fs(4))) min(fs(3),ss(3)) min(fs(4),ss(4))]);

searchpan = uipanel('units','normal','position',[0.01 .05 .38 .94],'title','Search','backgroundcolor','w');
bookpan = uipanel('units','normal','position',[.41 .05 .38 .94],'title','Bookmarks','backgroundcolor','w');
catpan = uipanel('units','normal','position',[.81 .05 .18 .94],'title','Catalogs','backgroundcolor','w');

% create disclaimer text and discontinued glasses checkbox
disctxt = uicontrol('style','text','string','Disclaimer: University of Rochester is not responsible for incorrect data','parent',figcon,'units','normal','position',[0.5 0 0.5 0.04]);
showdis = uicontrol('style','checkbox','string','Show Discontinued Glasses','parent',figcon,'units','normal','position',[0.05 0 0.3 0.04],'callback',@distoggle);

% search controls
fullname = cellfun(@(nam,cat) [nam '_' cat], name, catalog, 'uniformoutput', false);
indlist = [];
searchtxt = uicontrol('parent',searchpan,'style','edit','string','BA','units','normal','position',[0.1 0.85 0.8 0.1],'HorizontalAlignment','left','KeyPressFcn',@searchtype);
searchlist = uicontrol('parent',searchpan,'style','listbox','string','BA','units','normal','position',[0.1 0.05 0.8 0.75],'backgroundcolor','w','Callback',@searchselect);
searchtype(searchtxt,[]);

% bookmark controls
bkax = axes('parent',bookpan,'units','normal','position',[0 0.11 .99 .89],'xlim',[0 1],'ylim',[-0.5 0.5],'xtick',[],'ytick',[],'box','on','ydir','reverse','ButtonDownFcn',@bkselect);
set(bkax,'DefaulttextHorizontalAlignment','left','DefaulttextVerticalAlignment','middle','DefaulttextFontSize',10,'DefaulttextFontWeight','bold','DefaulttextInterpreter','none','DefaulttextHitTest','off');
bktx = 0.3;
bksx = bktx/2;
text(0.5,0,'No Bookmarks Selected','horizontalalignment','center');
bkind = [];
bksym = {};
bkcol = [];
bkclearall = uicontrol('parent',bookpan,'style','pushbutton','string','Clear All Bookmarks','units','normal','position',[0.05 0 .9 .1],'callback',@delbookmark);
markerspec = {'o';'+';'*';'.';'x';'s';'d';'^';'v';'>';'<';'p';'h'};
colorspec = hsv(6); %= get(bkax,'colororder');

%% index/abbe figure
fogm = findobj('type','figure','Name','Index vs. Abbe');
if isempty(fogm);
    figmap = figure(); % create figure for glassmaps
else
    figure(fogm(1));
    figmap = clf('reset'); % clear previous "Glassmap" figure
end
fs = [50 80 750 850];
set(figmap,'Toolbar','figure','CloseRequestFcn',@closefigs,'Name','Index vs. Abbe','numbertitle','off','position',[max(0,min(fs(1),ss(3)-fs(3))) max(0,min(fs(2),ss(4)-fs(4))) min(fs(3),ss(3)) min(fs(4),ss(4))]);

% create VIS plots%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
axdisp = zeros(2); % array of axes handles [ VIS abbe,  VIS partial  ;
                   %                         SWIR abbe, SWIR partial ]
axdisp(1,1) = axes('parent',figmap,'units','normal','position',[.1 0.55 .8 0.4]);
pvis = zeros(size(cats));
qvis = pvis;
pcolor = lines(numel(cats));
hold on;
for ci = 1:numel(cats)
    if isempty(catglasav{ci})
        pvis(ci) = plot(0,0,markerspec{ci},'color',pcolor(ci,:),'visible','off','HitTest','off');
        qvis(ci) = quiver(0,0,0,1,'AutoScale','off','color',pcolor(ci,:),'MaxHeadSize',0.001,'visible','off','HitTest','off');
    else
        pvis(ci) = plot(vd(catglasav{ci}),nd(catglasav{ci}),markerspec{ci},'color',pcolor(ci,:),'HitTest','off');
        qvis(ci) = quiver(vd(catglasav{ci}),nd(catglasav{ci}),vs(catglasav{ci})-vd(catglasav{ci}),ns(catglasav{ci})-nd(catglasav{ci}),...
            'AutoScale','off','color',pcolor(ci,:),'MaxHeadSize',0.001,'visible','off','HitTest','off'); % arrows pointing to location in other band (leave visible for now)
    end
end
hold off;
grid on;
xlabel('\nu_d');
ylabel('n_d');
title('VIS');
glab = zeros(2);
glab(1,1) = text(vd(curg),nd(curg),[name{curg} '\_' catalog{curg}],'visible','off','hittest','off','FontWeight','bold','VerticalAlignment','Top','interpreter','none'); % VIS glass label
legh = cLegend(pvis(catav), cats(catav), 'groups', 1:1:numel(catav), 'orientation', 'vertical', 'parent', catpan);
set(legh, 'units', 'normal', 'position', [0 0 1 1], 'box', 'off');
set(axdisp(1,1), 'ButtonDownFcn', @glabclick);

% create SWIR plot %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
axdisp(2,1) = axes('parent',figmap,'units','normal','position',[0.1 0.07 0.8 0.4]);
pswir = zeros(size(cats));
qswir = pswir;
hold on;
for cj = 1:numel(cats)
    if isempty(catglasav{cj})
        pswir(cj) = plot(0,0,markerspec{cj},'color',pcolor(cj,:),'visible','off','HitTest','off');
%         qswir(cj) = quiver(0,0,0,1,'AutoScale','off','color',pcolor(cj,:),'MaxHeadSize',0.001,'visible','off','HitTest','off');
    else
        pswir(cj) = plot(vs(catglasav{cj}),ns(catglasav{cj}),markerspec{cj},'color',pcolor(cj,:),'HitTest','off');
%         qswir(cj) = quiver(vs(catglasav{cj}),ns(catglasav{cj}),vd(catglasav{cj})-vs(catglasav{cj}),nd(catglasav{cj})-ns(catglasav{cj}),...
%             'AutoScale','off','color',pcolor(cj,:),'MaxHeadSize',0.001,'visible','off','HitTest','off'); % arrows pointing to location in other band (leave visible for now)
    end
end
hold off;
grid on;
xlabel('\nu_{0.9, 1.3, 1.7}');
ylabel('n_{1.3}');
title('SWIR');
glab(2,1) = text(vs(curg),ns(curg),[name{curg} '_' catalog{curg}],'visible','off','hittest','off','FontWeight','bold','VerticalAlignment','Top','interpreter','none'); % SWIR glass label
set(axdisp(2,1), 'ButtonDownFcn', @glabclick);

% create all interband lines
arrhan = zeros(numel(name),4); % arrow annotation handles, row # is glass index, column 1 is VIS, column 2 is SWIR, column 3 is VIS(part), column 2 is SWIR(part)
catnum = zeros(size(name));
for gi = 1:size(arrhan,1);
    catnum(gi) = find(strcmp(catalog{gi},cats),1);
    arrhan(gi,1) = line([vd(gi);vs(gi)],[nd(gi);ns(gi)],[0;0],'parent',axdisp(1,1),'HitTest','off',...
        'color',pcolor(catnum(gi),:)./max(pcolor(catnum(gi),:)).*abs(vs(gi)-vd(gi))./max(abs(vs(catglas{catnum(gi)})-vd(catglas{catnum(gi)}))));
    arrhan(gi,2) = line([vd(gi);vs(gi)],[nd(gi);ns(gi)],[0;0],'parent',axdisp(2,1),'HitTest','off',...
        'color',pcolor(catnum(gi),:)./max(pcolor(catnum(gi),:)).*abs(vs(gi)-vd(gi))./max(abs(vs(catglas{catnum(gi)})-vd(catglas{catnum(gi)}))));
    if ~av(gi)
        set(arrhan(gi,1:2),'visible','off');
    end
end

% % create interband arrows checkbox
% showarr = uicontrol('style','checkbox','string','Show Interband Vectors','parent',figmap,'position',[180,10,150,20],'callback',@arrtoggle);

isvis = arrayfun(@(cati) strcmp('on',get(pvis(cati),'visible')),(1:numel(cats)).');
set(pswir(isvis), 'HitTest','off','Visible','on','handlevisibility','on');
set(pswir(~isvis), 'HitTest','off','Visible','off','handlevisibility','off');

linkaxes([axdisp(1,1);axdisp(2,1)],'xy');
set(qvis,'visible','off');
% set(qswir,'visible','off');
set(arrhan,'visible','off');

%% create partial dispersion figure
fogp = findobj('type','figure','Name','Partial Dispersion');
if isempty(fogp);
    figpart = figure(); % create figure for Partial Dispersion plots
else
    figure(fogp(1));
    figpart = clf('reset'); % clear previous "Partial Dispersion" figure
end
fs = [100 50 750 850];
set(figpart,'Toolbar','figure','CloseRequestFcn',@closefigs,'Name','Partial Dispersion','numbertitle','off','position',[max(0,min(fs(1),ss(3)-fs(3))) max(0,min(fs(2),ss(4)-fs(4))) min(fs(3),ss(3)) min(fs(4),ss(4))]);

% create VIS plots %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
axdisp(1,2) = axes('parent',figpart,'units','normal','position',[.1 0.55 .8 0.4]);
parth = zeros(size(cats,1),2);
hold on;
for ci = 1:numel(cats)
    if isempty(catglasav{ci})
        parth(ci,1) = plot(0,0,markerspec{ci},'color',pcolor(ci,:),'visible','off','HitTest','off');
    else
        parth(ci,1) = plot(vd(catglasav{ci}),PdC(catglasav{ci}),markerspec{ci},'color',pcolor(ci,:),'HitTest','off');
    end
end
hold off;
grid on;
xlabel('\nu_d');
ylabel('P_{d,C}');
title('VIS');
glab(1,2) = text(vd(curg),PdC(curg),[name{curg} '_' catalog{curg}],'visible','off','hittest','off','FontWeight','bold','VerticalAlignment','Top','interpreter','none'); % VIS glass label
set(axdisp(1,2), 'ButtonDownFcn', @glabclick);

% create SWIR plot %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
axdisp(2,2) = axes('parent',figpart,'units','normal','position',[0.1 0.07 0.8 0.4]);
hold on;
for cj = 1:numel(cats)
    if isempty(catglasav{cj})
        parth(cj,2) = plot(0,0,markerspec{cj},'color',pcolor(cj,:),'visible','off','HitTest','off');
    else
        parth(cj,2) = plot(vs(catglasav{cj}),Pswir(catglasav{cj}),markerspec{cj},'color',pcolor(cj,:),'HitTest','off');
    end
end
hold off;
grid on;
xlabel('\nu_{0.9, 1.3, 1.7}');
ylabel('P_{1.3, 1.7}');
title('SWIR');
glab(2,2) = text(vs(curg),Pswir(curg),[name{curg} '_' catalog{curg}],'visible','off','hittest','off','FontWeight','bold','VerticalAlignment','Top','interpreter','none'); % SWIR glass label
set(axdisp(2,2), 'ButtonDownFcn', @glabclick);

% create all interband lines
for gi = 1:size(arrhan,1);
    arrhan(gi,3) = line([vd(gi);vs(gi)],[PdC(gi);Pswir(gi)],[0;0],'parent',axdisp(1,2),'visible','on','HitTest','off',...
        'color',pcolor(catnum(gi),:)./max(pcolor(catnum(gi),:)).*abs(vs(gi)-vd(gi))./max(abs(vs(catglas{catnum(gi)})-vd(catglas{catnum(gi)}))));
    arrhan(gi,4) = line([vd(gi);vs(gi)],[PdC(gi);Pswir(gi)],[0;0],'parent',axdisp(2,2),'visible','on','HitTest','off',...
        'color',pcolor(catnum(gi),:)./max(pcolor(catnum(gi),:)).*abs(vs(gi)-vd(gi))./max(abs(vs(catglas{catnum(gi)})-vd(catglas{catnum(gi)}))));
    if ~av(gi)
        set(arrhan(gi,3:4),'visible','off');
    end
end

set(parth(isvis,:), 'HitTest','off','Visible','on');
set(parth(~isvis,:), 'HitTest','off','Visible','off');

linkaxes([axdisp(1,2);axdisp(2,2)],'x');
set(arrhan,'visible','off');
rbglass = [];

%% create glass details figure
fogd = findobj('type','figure','Name','Glass Details');
if isempty(fogd);
    figdet = figure(); % create figure for glass details
else
    figure(fogd(1));
    figdet = clf('reset'); % clear previous "Glass Details" figure
end
fs = [880 50 650 450];
set(figdet,'visible','off','CloseRequestFcn',@closefigs,'Name','Glass Details','numbertitle','off','position',[max(0,min(fs(1),ss(3)-fs(3))) max(0,min(fs(2),ss(4)-fs(4))) min(fs(3),ss(3)) min(fs(4),ss(4))],'color','w');

[axdet,nline,tline] = plotyy(wavplot.*1000,nplot(curg,:),twav(curg,:),tdat(curg,:));
set(nline,'linewidth',3);
set(tline,'linewidth',3);
set(axdet,'units','normal','position',[0.1 0.15 0.6 0.75])
xlim(axdet(1),[450 1700]);
xlim(axdet(2),[450 1700]);
ylim(axdet(2),[0 1.1]);
xlabel(axdet(1),'Wavelength (nm)','FontWeight','bold');
ylabel(axdet(1),'Refractive Index','FontWeight','bold');
ylabel(axdet(2),'Internal Transmittance','FontWeight','bold');
nlims = ylim(axdet(1));
tlims = ylim(axdet(2));
set(axdet(1),'ytick',linspace(nlims(1),nlims(2),12));
set(axdet(2),'ytick',linspace(tlims(1),tlims(2),12));
grid(axdet(1),'on');
tiax = axes('parent',figdet,'units','normal','position',[0 0.95 0.8 0.05],'xlim',[0 1],'ylim',[0 1],'visible','off');
tidet = text(0.5,0.5,[name{curg} '_' catalog{curg}],'HorizontalAlignment','center','VerticalAlignment','Middle','FontSize',15,'FontWeight','bold','parent',tiax,'interpreter','none','hittest','off');
gdtxt = {sprintf('n_d = %0.3f\n', nd(curg));...
         sprintf('v_d = %0.1f\n', vd(curg));...
         sprintf('P_{d,C} = %0.3f\n\n', PdC(curg));...
         sprintf('n_{1.3} = %0.3f\n', ns(curg));...
         sprintf('v_{0.9, 1.3, 1.7} = %0.1f\n', vs(curg));...
         sprintf('P_{1.3, 1.7} = %0.3f\n\n', Pswir(curg))};
if av(curg) == 0
    gdtxt{end+1} = 'DISCONTINUED';
end
axdet2 = axes('units','normal','position',[0.75 0.4 0.2 0.45],'xlim',[0 1],'ylim',[0 1]);
axis off;
txdet = text(0.2,1,gdtxt,'VerticalAlignment','top','FontWeight','bold');
ndvd = line(wavcalc(1:3).*1000,ncalc(curg,1:3),'parent',axdet(1),'linestyle','none','marker','d','MarkerEdgeColor',[.5 0 0],'MarkerFaceColor',[.5 0 0]);
ndvdtxt = text(wavcalc(1:3).*1000,ncalc(curg,1:3),{'n_{F}','n_{d}','n_{C}'},'parent',axdet(1),'color',[.5 0 0],'fontweight','bold','VerticalAlignment','Bottom');
nsvs = line(wavcalc(4:6).*1000,ncalc(curg,4:6),'parent',axdet(1),'linestyle','none','marker','o','MarkerEdgeColor','k','MarkerFaceColor','k');
nsvstxt = text(wavcalc(4:6).*1000,ncalc(curg,4:6),{'n_{0.9}','n_{1.3}','n_{1.7}'},'parent',axdet(1),'color','k','fontweight','bold','VerticalAlignment','Bottom');

% create bookmark button and marker
bkbut = uicontrol('parent',figdet,'style','pushbutton','string','Bookmark Glass','units','normal','position',[0.8 0.95 0.2 0.05],'callback',@addbookmark);
marsiz = 14;
linwid = 3;
bksymbol = line('xdata',.95,'ydata',0.5,'visible','off','parent',tiax,'markersize',marsiz,'linewidth',linwid,'buttondownfcn',@changemarker);

%% create marker selection figure
fogm = findobj('type','figure','Name','Marker Selection');
if isempty(fogm);
    figmar = figure(); % create figure for glassmaps
else
    figure(fogm(1));
    figmar = clf('reset'); % clear previous "Marker Selection" figure
end
fs = [200 200 300 400];
set(figmar,'Toolbar','none','CloseRequestFcn',@closefigs,'visible','off','Name','Marker Selection','position',[max(0,min(fs(1),ss(3)-fs(3))) max(0,min(fs(2),ss(4)-fs(4))) min(fs(3),ss(3)) min(fs(4),ss(4))]);

% create axes
axsel = axes('parent',figmar,'units','normal','position',[.1 0.9 .8 .1],'xlim',[0 1],'ylim',[0 1],'visible','off');
axsym = axes('parent',figmar,'units','normal','position',[.2 0.1 .2 .8],'xlim',[0 1],'ylim',[0.5 numel(markerspec)+0.5],'xtick',[],'ytick',[],'box','on','ydir','reverse','ButtonDownFcn',@marselect);
axcol = axes('parent',figmar,'units','normal','position',[.6 0.2 .2 .7],'xlim',[0 1],'ylim',[0.5 size(colorspec,1)+0.5],'xtick',[],'ytick',[],'box','on','ydir','reverse','ButtonDownFcn',@marselect);

% populate selection axis
seldti = text(0.05,0.5,'Selection:','fontsize',12,'fontweight','bold','parent',axsel,'interpreter','none');
seldmar = line('xdata',0.85,'ydata',0.5,'markersize',marsiz,'linewidth',linwid,'parent',axsel);

% create buttons
chmar = [];
colbut = uicontrol('style','pushbutton','parent',figmar,'units','normal','position',[.6  0.11 .2 .08],'string','Custom','callback',@custcol);
selbut = uicontrol('style','pushbutton','parent',figmar,'units','normal','position',[.55 0.01 .4 .08],'string','Select','callback',@doneselect);
canbut = uicontrol('style','pushbutton','parent',figmar,'units','normal','position',[.05 0.01 .4 .08],'string','Cancel','callback',@doneselect);

% populate symbol axis
for si = 1:numel(markerspec);
    line('xdata',0.5,'ydata',si,'marker',markerspec{si},'color','k','markersize',marsiz,'linewidth',linwid,'parent',axsym,'hittest','off');
end

% populate color axis
for ci = 1:size(colorspec,1)
    patch([0;1;1;0],[ci-0.5;ci-0.5;ci+0.5;ci+0.5],ones(4,1),'edgecolor','k','facecolor',colorspec(ci,:),'hittest','off');
end

%% set up bookmarks
bknum = 20;
bklabh = zeros(bknum,6); % bookmark label handles [ list name, list marker, vis abbe marker, swir abbe marker, vis partial marker, swir partial marker ]
for bi = 1:bknum;
    [marsty0,marcol] = randmar(1);
    marsty = marsty0{1};
    bklabh(bi,1) = text(bktx,bi,'unspecified','parent',bkax);
    bklabh(bi,2) = line('xdata',bksx,'ydata',bi,'marker',marsty,'color',marcol,'markersize',marsiz,'linewidth',linwid,'parent',bkax,'hittest','off');
    bklabh(bi,3) = line('xdata',0,'ydata',0,'marker',marsty,'color',marcol,'markersize',marsiz,'linewidth',linwid,'parent',axdisp(1,1),'visible','off','hittest','off');
    bklabh(bi,4) = line('xdata',0,'ydata',0,'marker',marsty,'color',marcol,'markersize',marsiz,'linewidth',linwid,'parent',axdisp(2,1),'visible','off','hittest','off');
    bklabh(bi,5) = line('xdata',0,'ydata',0,'marker',marsty,'color',marcol,'markersize',marsiz,'linewidth',linwid,'parent',axdisp(1,2),'visible','off','hittest','off');
    bklabh(bi,6) = line('xdata',0,'ydata',0,'marker',marsty,'color',marcol,'markersize',marsiz,'linewidth',linwid,'parent',axdisp(2,2),'visible','off','hittest','off');
end

%% showpath function
    function [thePath] = ShowPath()
        % Show EXE path:
        if isdeployed % Stand-alone mode.
            [status, result] = system('set PATH');
            thePath = char(regexpi(result, 'Path=(.*?);', 'tokens', 'once'));
        else % Running from MATLAB.
            [macroFolder, baseFileName, ext] = fileparts(mfilename('fullpath'));
            thePath = macroFolder;
            % thePath = pwd;
        end
    end
%% clickable legend function
    function varargout = cLegend(varargin) % replica of "clickableLegend.m" so we have access to callback
        % Extract any arguments for clickableLegend
        ind = find(strcmpi(varargin,'DisplayedLines'));
        if ~isempty(ind)
            if ind == nargin
                error('The DisplayedLines parameter value must be specified');
            end
            dispflag = true;
            dispinds = varargin{ind+1};
            varargin(ind:ind+1) = [];
        else
            dispflag = false;
        end
        
        ind = find(strcmpi(varargin,'groups'));
        if ~isempty(ind)
            if ind == nargin
                error('The groups parameter value must be specified');
            end
            groupflag = true;
            groupmem = varargin{ind+1};
            varargin(ind:ind+1) = [];
            
            groups = unique(groupmem);
            firstmem = zeros(size(groups));
            if nargin > 0 && ishandle(varargin{1}(1))
                if strcmpi(get(varargin{1}(1),'Type'),'axes')
                    hAxes = varargin{1}(1);
                    obj = flipud([findobj(hAxes,'Type','line');findobj(hAxes,'Type','patch')]);
                else % It's a line/patch
                    obj = varargin{1};
                    for i = 1:length(groups)
                        firstmem(i) = find(groupmem==groups(i),1);
                    end
                    varargin{1} = obj(firstmem);
                end
            else
                hAxes = gca;
                obj = flipud([findobj(hAxes,'Type','line');findobj(hAxes,'Type','patch')]);
            end
        else
            groupflag = false;
        end
        
        % Create legend as if it was called directly
        [varargout{1:nargout(@legend)}] = legend(varargin{:});
        
        % Extract what is needed for the rest of the function and fix varargout
        [leghan, objhan, plothan] = varargout{1:3};
        % objhan: strings
        % plothan: graphics objects
        varargout = varargout(1:nargout);
        
        % At this point, we can quit and behavior would be equivalent to calling
        % just legend.
        
        if ~groupflag % Default group membership
            groupmem = 1:length(plothan);
            obj = plothan;
            groups = groupmem;
        end
        
        if dispflag % DisplayedLines parameter was passed in
            hidden = true(1, length(plothan));
            dispinds(dispinds>length(plothan)) = [];
            hidden(dispinds) = false;
        end
        
        % Set the callbacks
        for i = 1:length(plothan)
            set(objhan(i), 'HitTest', 'on', 'ButtonDownFcn',...
                @(varargin)togglevisibility(objhan(i),obj(groupmem==groups(i))),...
                'UserData', true);
            if dispflag && hidden(i)
                togglevisibility(objhan(i), obj(groupmem==groups(i)));
            end
        end
    end
%% visibility toggle function
    function togglevisibility(hObject, obj)
        if get(hObject, 'UserData') % It is on, turn it off
            set(hObject, 'Color', (get(hObject, 'Color') + 1)/1.5, 'UserData', false);
            set(obj,'HitTest','off','Visible','off','handlevisibility','off');
        else
            set(hObject, 'Color', get(hObject, 'Color')*1.5 - 1, 'UserData', true);
            set(obj, 'HitTest','off','visible','on','handlevisibility','on');
        end
        isvis = arrayfun(@(cati) strcmp('on',get(pvis(cati),'visible')),(1:numel(cats)).');
        set(pswir(isvis), 'Visible','on');
        set(pswir(~isvis), 'Visible','off');
        set(parth(isvis,:), 'visible','on');
        set(parth(~isvis,:), 'visible','off');
        if disyn
            rbglass = intersect(rbglass,cell2mat(catglas(isvis)));
        else
            rbglass = intersect(rbglass,cell2mat(catglasav(isvis)));
        end
        set(arrhan,'visible','off');
        set(arrhan(rbglass,:),'visible','on');
    end
%% discontinued glass toggle function
    function distoggle(hObject, eventdata, handles)
        if (get(hObject,'Value') == get(hObject,'Max')) % selected
            for ck = 1:numel(cats)
                if isempty(catglas{ck})
                    set(pvis(ck), 'xdata', 0, 'ydata', 0, 'visible', 'off');
                    set(pswir(ck), 'xdata', 0, 'ydata', 0, 'visible', 'off');
%                     set(qvis(ck), 'xdata', 0, 'ydata', 0, 'udata', 0, 'vdata', 1, 'visible', 'off');
%                     set(qswir(ck), 'xdata', 0, 'ydata', 0, 'udata', 0, 'vdata', 1, 'visible', 'off');
                else
                    set(pvis(ck), 'xdata', vd(catglas{ck}), 'ydata', nd(catglas{ck}), 'visible', 'on');
                    set(pswir(ck), 'xdata', vs(catglas{ck}), 'ydata', ns(catglas{ck}), 'visible', 'on');
%                     set(qvis(ck), 'xdata', vd(catglas{ck}), 'ydata', nd(catglas{ck}),...
%                         'udata', vs(catglas{ck})-vd(catglas{ck}), 'vdata', ns(catglas{ck})-nd(catglas{ck}));
%                     set(qswir(ck), 'xdata', vs(catglas{ck}), 'ydata', ns(catglas{ck}),...
%                         'udata', vd(catglas{ck})-vs(catglas{ck}), 'vdata', nd(catglas{ck})-ns(catglas{ck}));
                end
            end
            axes(axdisp(1,1));
            delete(legh);
            legh = cLegend(pvis, cats, 'groups', 1:1:numel(cats), 'location', 'southoutside', 'orientation', 'vertical', 'parent', catpan);
            set(legh, 'units', 'normal', 'position', [0 0 1 1], 'box', 'off');
            isvis = arrayfun(@(cati) strcmp('on',get(pvis(cati),'visible')),(1:numel(cats)).');
            disyn = true;
            rbglass = intersect(rbglass,cell2mat(catglas(isvis)));
        else % not selected
            for ck = 1:numel(cats)
                if isempty(catglasav{ck})
                    set(pvis(ck), 'xdata', 0, 'ydata', 0, 'visible', 'off');
                    set(pswir(ck), 'xdata', 0, 'ydata', 0, 'visible', 'off');
%                     set(qvis(ck), 'xdata', 0, 'ydata', 0, 'udata', 0, 'vdata', 1, 'visible', 'off');
%                     set(qswir(ck), 'xdata', 0, 'ydata', 0, 'udata', 0, 'vdata', 1, 'visible', 'off');
                else
                    set(pvis(ck), 'xdata', vd(catglasav{ck}), 'ydata', nd(catglasav{ck}), 'visible', 'on');
                    set(pswir(ck), 'xdata', vs(catglasav{ck}), 'ydata', ns(catglasav{ck}), 'visible', 'on');
%                     set(qvis(ck), 'xdata', vd(catglasav{ck}), 'ydata', nd(catglasav{ck}),...
%                         'udata', vs(catglasav{ck})-vd(catglasav{ck}), 'vdata', ns(catglasav{ck})-nd(catglasav{ck}));
%                     set(qswir(ck), 'xdata', vs(catglasav{ck}), 'ydata', ns(catglasav{ck}),...
%                         'udata', vd(catglasav{ck})-vs(catglasav{ck}), 'vdata', nd(catglasav{ck})-ns(catglasav{ck}));
                end
            end
            axes(axdisp(1,1));
            delete(legh);
            legh = cLegend(pvis(catav), cats(catav), 'groups', 1:1:numel(catav), 'location', 'southoutside', 'orientation', 'vertical', 'parent', catpan);
            set(legh, 'units', 'normal', 'position', [0 0 1 1], 'box', 'off');
            isvis = arrayfun(@(cati) strcmp('on',get(pvis(cati),'visible')),(1:numel(cats)).');
            disyn = false;
            rbglass = intersect(rbglass,cell2mat(catglasav(isvis)));
        end
%         if (get(showarr,'Value') == get(showarr,'Max')) % arrow toggle is selected
%             set([qvis(isvis);qswir(isvis)], 'visible', 'on');
%         else
%             set([qvis;qswir], 'visible', 'off');
%         end
        set(arrhan,'visible','off');
        set(arrhan(rbglass,:),'visible','on');
        drawnow expose;
    end
%% arrow toggle function (not used)
%     function arrtoggle(hObject, ~, ~)
%         if (get(hObject,'Value') == get(hObject,'Max')) % selected
% %             set(qvis(isvis),'visible','on');
% %             set(qswir(isvis),'visible','on');
%             set(arrhan(rbglass,:),'visible','on');
%         else % not selected
% %             set(qvis,'visible','off');
% %             set(qswir,'visible','off');
%             set(arrhan,'visible','off');
%         end
%     end
%% glass details selection function
    function glabclick(axhan,~)
        point1 = get(axhan,'CurrentPoint'); % axhax is handle to axes
        rbbox;
        point2 = get(axhan,'CurrentPoint'); % axhax is handle to axes
        pos = getpixelposition(axhan,true);
        xl = get(axhan,'xlim');
        yl = get(axhan,'ylim');
        pxpv = pos(3)/diff(xl);
        pxpn = pos(4)/diff(yl);
        pt1 = point1(1,1:2);
        pt2 = point2(1,1:2);
        cpt1 = pt1.*[pxpv,pxpn];
        cpt2 = pt2.*[pxpv,pxpn];
%         disp([cpt1;cpt2]);
        if disyn
            curvis = cell2mat(catglas(isvis)); % indices of all currently visible glasses
        else
            curvis = cell2mat(catglasav(isvis)); % indices of all currently visible glasses
        end
        if axhan == axdisp(1,1)
            curns = nd(curvis);
            curvs = vd(curvis);
        elseif axhan == axdisp(2,1)
            curns = ns(curvis);
            curvs = vs(curvis);
        elseif axhan == axdisp(1,2)
            curns = PdC(curvis);
            curvs = vd(curvis);
        elseif axhan == axdisp(2,2)
            curns = Pswir(curvis);
            curvs = vs(curvis);
        end
        if all(pt1 == pt2);
            dist = (curvs.*pxpv-cpt1(1)).^2 + (curns.*pxpn-cpt1(2)).^2;
            [~,mind] = min(dist(:));
            ming = curvis(mind); % index of currently visible glass with shortest distance from clicked point
            if isempty(curvis)
                set(glab,'visible','off');
                set(figdet,'visible','off');
                glabyn = false;
            elseif glabyn && ming==curg; % closest glass already labeled
                set(glab,'visible','off');
                set(figdet,'visible','off');
                glabyn = false;
            else
                updatecurg(ming);
%                 curg = ming;
%                 set(glab(1,1),'position',[vd(curg),nd(curg)],'string',[name{curg} '\_' catalog{curg}],'visible','on');
%                 set(glab(2,1),'position',[vs(curg),ns(curg)],'string',[name{curg} '\_' catalog{curg}],'visible','on');
%                 set(glab(1,2),'position',[vd(curg),PdC(curg)],'string',[name{curg} '\_' catalog{curg}],'visible','on');
%                 set(glab(2,2),'position',[vs(curg),Pswir(curg)],'string',[name{curg} '\_' catalog{curg}],'visible','on');
%                 set(tidet,'string',[name{curg} '_' catalog{curg}]);
%                 newtxt = {sprintf('n_d = %0.3f\n', nd(curg));...
%                     sprintf('v_d = %0.1f\n', vd(curg));...
%                     sprintf('P_{d,C} = %0.3f\n\n', PdC(curg));...
%                     sprintf('n_{1.3} = %0.3f\n', ns(curg));...
%                     sprintf('v_{1.0, 1.3, 1.6} = %0.1f\n', vs(curg));...
%                     sprintf('P_{1.3, 1.6} = %0.3f\n\n', Pswir(curg))};
%                 if av(curg) == 0
%                     newtxt{end+1} = 'DISCONTINUED';
%                 end
%                 set(txdet,'string',newtxt);
%                 set(figdet,'visible','on');
%                 set(nline,'ydata',nplot(curg,:));
%                 set(tline,'xdata',twav(curg,:),'ydata',tdat(curg,:));
%                 nhi = nanmax(nplot(curg,:));
%                 nlo = nanmin(nplot(curg,:));
%                 ylim(axdet(1),[nlo-0.1*(nhi-nlo), nhi+0.1*(nhi-nlo)]);
%                 set(axdet(1),'ytick',linspace(nlo-0.1*(nhi-nlo),nhi+0.1*(nhi-nlo),12));
%                 set(ndvd, 'ydata', ncalc(curg,1:3));
%                 set(nsvs, 'ydata', ncalc(curg,4:6));
%                 for dum = 1:3
%                     set(ndvdtxt(dum), 'position', [wavcalc(dum)*1000,ncalc(curg,dum)]);
%                     set(nsvstxt(dum), 'position', [wavcalc(dum+3)*1000,ncalc(curg,dum+3)]);
%                 end
%                 glabyn = true;
            end
        else % if mouse down and mouse up occur at different points
            rbglass = curvis(curns>=min([pt1(2),pt2(2)]) & curns<=max([pt1(2),pt2(2)]) & curvs>=min([pt1(1),pt2(1)]) & curvs<=max([pt1(1),pt2(1)]));
            set(arrhan,'visible','off');
            set(arrhan(rbglass,:),'visible','on');
            % bold the one on the labeled glass?
        end
    end
%% update current glass and show glass details figure
    function updatecurg(newcurg)
        curg = newcurg;
        % glass label positions and text
        gnamtxt = fullname{curg};
        set(glab(1,1),'position',[vd(curg),nd(curg)],'string',gnamtxt,'visible','on');
        set(glab(2,1),'position',[vs(curg),ns(curg)],'string',gnamtxt,'visible','on');
        set(glab(1,2),'position',[vd(curg),PdC(curg)],'string',gnamtxt,'visible','on');
        set(glab(2,2),'position',[vs(curg),Pswir(curg)],'string',gnamtxt,'visible','on');
        % glass details title
        set(tidet,'string',gnamtxt);
        % glass details text
        newtxt = {sprintf('n_d = %0.3f\n', nd(curg));...
            sprintf('v_d = %0.1f\n', vd(curg));...
            sprintf('P_{d,C} = %0.3f\n\n', PdC(curg));...
            sprintf('n_{1.3} = %0.3f\n', ns(curg));...
            sprintf('v_{0.9, 1.3, 1.7} = %0.1f\n', vs(curg));...
            sprintf('P_{1.3, 1.7} = %0.3f\n\n', Pswir(curg))};
        if av(curg) == 0
            newtxt{end+1} = 'DISCONTINUED';
        end
        set(txdet,'string',newtxt);
        % make glass details figure visible
        set(figdet,'visible','on');
        % update index/transmission plot
        set(nline,'ydata',nplot(curg,:));
        set(tline,'xdata',twav(curg,:),'ydata',tdat(curg,:));
        nhi = nanmax(nplot(curg,:));
        nlo = nanmin(nplot(curg,:));
        ylim(axdet(1),[nlo-0.1*(nhi-nlo), nhi+0.1*(nhi-nlo)]);
        set(axdet(1),'ytick',linspace(nlo-0.1*(nhi-nlo),nhi+0.1*(nhi-nlo),12));
        set(ndvd, 'ydata', ncalc(curg,1:3));
        set(nsvs, 'ydata', ncalc(curg,4:6));
        for dum = 1:3
            set(ndvdtxt(dum), 'position', [wavcalc(dum)*1000,ncalc(curg,dum)]);
            set(nsvstxt(dum), 'position', [wavcalc(dum+3)*1000,ncalc(curg,dum+3)]);
        end
        if ismember(curg,bkind)
            set(bkbut,'string','Remove Bookmark','callback',@delbookmark);
            set(bksymbol,'visible','on','marker',bksym{bkind==curg},'color',bkcol(bkind==curg,:));
        else
            set(bkbut,'string','Bookmark Glass','callback',@addbookmark);
            set(bksymbol,'visible','off');
        end
        % set glass label visibility flag to true
        glabyn = true;
    end
%% close program function
    function closefigs(figh,~)
        button = questdlg('Close glass maps?','Close','Yes');
        if strcmpi(button,'yes');
            try
                delete(figmap);
            catch
            end
            try
                delete(figdet);
            catch
            end
            try
                delete(figpart);
            catch
            end
            try
                delete(figcon);
            catch
            end
            try
                delete(figmar);
            catch
            end
        end
    end
%% search edit box callback
    function searchtype(src,~)
        import java.awt.Robot;
        import java.awt.event.KeyEvent;
        robot=Robot;
        robot.keyPress(KeyEvent.VK_ENTER);
        pause(0.02)
        robot.keyRelease(KeyEvent.VK_ENTER);
        searchterm = upper(get(src,'String'));
        % create new list
        if isempty(searchterm)
            indlist = (1:size(fullname,1)).';
        else
            indlist0 = strfind(fullname,searchterm);
            indlist00 = find(~cellfun(@isempty,indlist0));
            firstocc = cellfun(@min,indlist0(indlist00));
            [~,slist] = sort(firstocc);
            indlist = indlist00(slist);
        end
        if isempty(indlist)
            newlist = '';
        else
            newlist = fullname(indlist);
        end
        % update search list
        set(searchlist,'string',newlist);
    end
%% search list mouse click callback
    function searchselect(src,~)
        if isempty(indlist)
        else
            newcurg = indlist(get(src,'value'));
            updatecurg(newcurg);
        end
    end
%% random marker generator function
    function [markersty, markercolr] = randmar(num)
        randinds1 = ceil(rand(num,1).*numel(markerspec));
        randinds2 = ceil(rand(num,1).*size(colorspec,1));
        markersty = markerspec(randinds1);
        markercolr = colorspec(randinds2,:);
    end
%% add bookmark function
    function addbookmark(src,~,varargin)
        if nargin>2
            glassinds = varargin{1};
        else
            glassinds = curg;
        end
        glassinds = setdiff(glassinds,bkind);
        bkind = [bkind;glassinds(:)];
        [ms,mc] = randmar(numel(glassinds));
        bksym = [bksym;ms(:)];
        bkcol = [bkcol;mc];
        bkupdate();
        if src == bkbut
            set(bkbut,'string','Remove Bookmark','callback',@delbookmark);
            set(bksymbol,'visible','on','marker',bksym{bkind==curg},'color',bkcol(bkind==curg,:));
        end
    end
%% delete all bookmarks function
    function delbookmark(src,~)
        if src == bkclearall
            keepind = [];
        elseif src == bkbut
            [~,keepind] = setdiff(bkind,curg);
            set(bkbut,'string','Bookmark Glass','callback',@addbookmark);
            set(bksymbol,'visible','off');
        end
        keepind = sort(keepind);
        bkind = bkind(keepind);
        bksym = bksym(keepind);
        bkcol = bkcol(keepind,:);
        bkupdate();
        if strcmp(get(figdet,'visible'),'on')
            updatecurg(curg);
        end
    end
%% update bookmarks function
    function bkupdate()
        if isempty(bkind)
            set(bkax,'ylim',[-0.5 0.5]);
            set(bklabh(:,3:end),'visible','off');
        else
            set(bkax,'ylim',[0 numel(bkind)]+0.5);
            for bj = 1:numel(bkind)
                set(bklabh(bj,1),'string',fullname(bkind(bj)));
                set(bklabh(bj,2:end),'marker',bksym{bj},'color',bkcol(bj,:));
                set(bklabh(bj,3),'xdata',vd(bkind(bj)),'ydata',nd(bkind(bj)));
                set(bklabh(bj,4),'xdata',vs(bkind(bj)),'ydata',ns(bkind(bj)));
                set(bklabh(bj,5),'xdata',vd(bkind(bj)),'ydata',PdC(bkind(bj)));
                set(bklabh(bj,6),'xdata',vs(bkind(bj)),'ydata',Pswir(bkind(bj)));
            end
            set(bklabh(:,3:end),'visible','off');
            set(bklabh(1:numel(bkind),3:end),'visible','on');
        end
    end
%% bookmark selection function
    function bkselect(src,~)
        point1 = get(src,'CurrentPoint'); % src is handle to axes
        bkindex = round(point1(1,2));
        if bkindex>=1
            newcurg = bkind(bkindex);
            updatecurg(newcurg);
        end
    end
%% initiate marker selection function
    function changemarker(src,~)
        chmar = curg;
        set(seldmar,'marker',get(bksymbol,'marker'),'color',get(bksymbol,'color'));
        set(seldti,'string',fullname{chmar});
        set(figmar,'visible','on');
    end
%% symbol and color selection callback
    function marselect(src,~)
        point1 = get(src,'CurrentPoint'); % src is handle to axes
        sindex = round(point1(1,2));
        switch src
            case axsym
                set(seldmar,'marker',markerspec{sindex});
            case axcol
                set(seldmar,'color',colorspec(sindex,:));
        end
    end
%% terminate selection callback
    function doneselect(src,~)
        if src==selbut
            bkindex = find(bkind==chmar);
            bksym{bkindex} = get(seldmar,'marker');
            bkcol(bkindex,:) = get(seldmar,'color');
            bkupdate();
            updatecurg(chmar);
        end
        set(figmar,'visible','off');
    end
%% custom color selection callback
    function custcol(~,~)
        uisetcolor(seldmar);
    end
%%
% save_to_base(1);
figure(figmap);
figure(figcon);
end









































